/* compiler.c++ -- written by Alexis WILKE for Made to Order Software Corp. (c) 2005-2009 */

/*

Copyright (c) 2005-2009 Made to Order Software Corp.

Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and
associated documentation files (the "Software"), to
deal in the Software without restriction, including
without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the
following conditions:

The above copyright notice and this permission notice
shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF
ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

*/

#include	"assembler.h"


namespace sswf
{
namespace asas
{
//using namespace as;




//**********************************************************************
//***  PUBLIC COMPILER  ************************************************
//**********************************************************************
Compiler::Compiler(void)
{
	f_options = &f_default_options;
	f_input_retriever = 0;
}

Compiler::~Compiler()
{
}




void Compiler::SetInputRetriever(as::InputRetriever& input)
{
	f_input_retriever = &input;
}


void Compiler::SetOptions(as::Options& options)
{
	f_options = &options;
}


int Compiler::Compile(as::NodePtr& program, const char *code, unsigned long line, const char *filename)
{
	as::String wcode;
	wcode.FromUTF8(code, strlen(code));
	as::StringInput input(filename);
	input.Set(wcode.Get(), wcode.GetLength(), line);
	f_options->SetOption(as::AS_OPTION_EXTENDED_ESCAPE_SEQUENCES, 1);
	f_options->SetOption(as::AS_OPTION_EXTENDED_OPERATORS, 1);
	f_options->SetOption(as::AS_OPTION_EXTENDED_STATEMENTS, 1);
	as::Parser *parser = as::Parser::CreateParser();
	parser->SetOptions(*f_options);
	parser->SetInput(input);
	program = parser->Parse();
	int errcnt = input.ErrCount();
	as::Compiler *compiler = as::Compiler::CreateCompiler(f_input_retriever);
	compiler->SetOptions(*f_options);
//fprintf(stderr, "] Compile... (Lexer result: %d)\n", errcnt);
	errcnt += compiler->Compile(program);
	as::Optimizer *optimizer = as::Optimizer::CreateOptimizer();
	optimizer->SetOptions(*f_options);
//fprintf(stderr, "] Optimize... (Compile result: %d)\n", errcnt);
	errcnt += optimizer->Optimize(program);
//fprintf(stderr, "] All done... (Optimizer result: %d)\n", errcnt);
	return errcnt;
}







};	// namespace asas
};	// namespace sswf
